static char rcsid[] = "$Id: xprstuff.c,v 1.14 1995/04/17 22:00:45 jrh Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <things.h>

#include <qdos.h>
#include "wild.h"

#include "xprdefs.h"

#include <xproto.h>
#include "xprmsg.h"

#define PIPEOK(p1) (p1) > 0

int XPRPid;
char MyProtocol[20] = {0};

extern PXPRTHG pXB;
static int IPipe = -1;
static int OPipe = -1;

char *stpcpy (char *d, const char *s)
{
    while (*d++ = *s++) ;
    return (d - 1);
}

static int myaccess (char *p, int mode)
{
    int cn;
    int rv = -1;
    struct qdirect buf;

    if ((cn = io_open (p, 0)) > 0)
    {
	if (mode == 1)
	{
	    if (fs_headr (cn, -1, &buf, sizeof (buf)) > 0)
	    {
		if (buf.d_type)
		{
		    rv = 0;
		}
	    }
	}
	else
	{
	    rv = 0;
	}
	io_close (cn);
    }
    return rv;
}

char *FindXPRExe (char *pName)
{
    static char ExeName[64];
    char *ePtr;
    int rv;

    ePtr = getenv ("XPR_PATH");
    
    if(ePtr == NULL)
    {
	if (*LibDir)
	{
	    ePtr = LibDir;
	}
	else
	{
	    ePtr = "";
	}
    }

    sprintf (ExeName, "%sxpr%s_library", ePtr, pName);
    if (rv = myaccess (ExeName, 1))
    {
	sprintf (ExeName, "%sxpr%s.library", ePtr, pName);
	rv = myaccess (ExeName, 1);
    }

    if (rv == 0)
    {
	return ExeName;
    }
    else
    {
	return NULL;
    }
}

void XPRKill (void)
{
    if (XPRPid > 0)
	mt_frjob (XPRPid, -1);
    if (IPipe > 0)
	io_close (IPipe);
    if (OPipe > 0)
	io_close (OPipe);
    IPipe = OPipe = XPRPid = 0;
}

XPRUse (char *pName)
{
    int n, val = 0;
    SJOB s;
    short msgno;
    int xstat;

    memset (&s, '\0', sizeof (SJOB));

    if (pName && *pName)
    {

        char mbuf[24];
        sprintf(mbuf, "%x\r", pXB);
	OPipe = io_open ("pipe_16", 0);

	s.FileName = FindXPRExe (pName);

	s.ComStr = mbuf;
	s.InFile = "pipe_0";
	s.inflag = OPipe;
	s.OuFile = "pipe_16";
	s.ouid = 3;
	s.serr = pXB->chid;
	s.Wait = 0;
	s.Prio = 32;
	s.JobId = -1;

	if ((xstat = spawn (&s)) == 0)
	{
	    long x;

	    XPRPid = s.JobId;
	    IPipe = io_open ("pipe_0", s.ouid);

	    msgno = XMSG_START;

	    io_sstrg (OPipe, -1, (char *) &msgno, 2);
	    n = io_fstrg (IPipe, -1, (char *) &x, 4);

#define BASE_XPR "3.49"

	    if (x >= *(long *) BASE_XPR && n == 4)
	    {
		val = pXB->status;
	    }
	    else
	    {
		xstat = -1;
	    }
	}

	if (xstat != 0)
	{
	    XPRKill ();
	    *MyProtocol = '\0';
	    val = 0;
	}
    }
    return val;
}

int XPRTidy (int res, int n)
{

    if (n <= 0)
    {
	XPRKill ();
	*MyProtocol = '\0';
	res = -1;
    }
    return res;
}
static char *ExtractFile (char *pat, short *pflag, short *recurse)
{
    static int fchid = 0;
    char *p, *q;
    static char text[64];

    if (fchid > 0)
    {
	int n;

	n = io_fline (fchid, -1, text, 64);
	if (n > 0)
	{
	    if ((p = strchr ((q = text), '\n')))
	    {
		*p = '\0';
	    }
	    if (*text == '-' && *(text + 1) == 'r')
	    {
		*recurse = 1;
		q = stpblk (text + 2);
	    }
	}
	else
	{
	    io_close (fchid);
	    fchid = 0;
	    /*
	     * Recursion: see recursion etc
	     */
	    q = ExtractFile (pat, pflag, recurse);
	}
    }
    else
    {
	q = strtok ((*pflag) ? NULL : pat, " ,\t");
	(*pflag)++;
	if (q && *q == '@')
	{
	    fchid = io_open (q + 1, 0);
	    q = ExtractFile (pat, pflag, recurse);
	}
	else if (q && *q == '-' && *(q + 1) == 'r')
	{
	    *recurse = 1;
	    q = ExtractFile (pat, pflag, recurse);
	}
    }
    return q;
}

static FLIST *xfl = NULL;

int MakeFileList (char *pat)
{
    static char *wild = "*?!";
    short recurse;
    char *p, *q;
    int n, totl = 0;
    short Done, k;

    if (xfl)
    {
	FreeWildList (&xfl);
    }

    for (Done = k = 0; !Done;)
    {
	recurse = 0;
	if ((p = ExtractFile (pat, &k, &recurse)) == NULL)
	{
	    Done = 1;
	    continue;
	}
	else if (strpbrk (p, "*?") == NULL)
	{
	    AddFile (&xfl, p);
	    totl++;
	}
	else
	{
	    n = ScanDir (p, &xfl, recurse);
	    totl += n;
	}
    }
    return totl;
}

int InitXIO (void *ptr, int IPipe, int OPipe, int dirn)
{
    int n;
    short msgno;

    pXB->buf = ptr;
    pXB->job = -1;
    pXB->status = -1;
    
    msgno = dirn;
    mt_prior(-1, 1);

    n = io_sstrg (OPipe, -1, (char *) &msgno, 2);
    if(n > 0)
    {
        pXB->flag = 3;
        pXB->icon = 1;
        n = io_fstrg (IPipe, -1, (char *) &msgno, 2);
    }
    return XPRTidy (pXB->status, n);
}

int XPRIO (void *f, int dirn)
{
    int msg, flag;
    short sx, sy;
    int n = -1;
    
    if (XPRPid > 0)
    {
	if (dirn == SEND || dirn == RECEIVE)
	{
	    if (dirn == SEND)
	    {
		if (f == NULL)
		{
		    f = xfl;
		}
		msg = XMSG_SEND;
	    }
	    else
	    {
		msg = XMSG_RECV;
	    }
	    n = InitXIO (f, IPipe, OPipe, msg);
	}
    }
    if (xfl)
    {
	FreeWildList (&xfl);
    }
    return n;
}

short destack(char **psp)
{
    char *p1;
    short dirn;
    
    if(p1 = strtok(*psp, " "))
    {
        if(*p1 == 's' || *p1 == 'S')
        {
            dirn = SEND;
        }
        else if(*p1 == 'r' || *p1 == 'R')
        {
            dirn = RECEIVE;
        }
        if(dirn && (p1 = strtok(NULL, " ")))
        {
            switch(*p1 | ' ')
            {
                case 'x':
                    strcpy(MyProtocol, "xmodem");
                    break;
                case 'y':
                    strcpy(MyProtocol, "ymodem");
                    break;
                case 'z':
                    strcpy(MyProtocol, "zmodem");
                    break;
                case 'a':
                    strcpy(MyProtocol, "ascii");
                    break;
                case 's':
                    strcpy(MyProtocol, "sealink");
                    break;
                case 'k':
                    strcpy(MyProtocol, "kermit");
                    break;
                default:
                    *MyProtocol = 0;
                    dirn = 0;
                    break;
            }
        }
        if(dirn && (p1 = strtok(NULL, " ")))
        {
            *psp = p1;
        }
    }
    return dirn;
}

extern void * GetXPRData (void);

int main (int ac, char **av)
{
    char *sp;
    char *msp;
    short nc;
    short dirn;
    int res = 0;
    void *p;

    if(GetXPRData())
    {

        sp = *(av + 1);		                /* supplied sp */
        nc = *((short *) sp)++;	                /* channels */

        if (nc > 1)
        {
            int dummy;
	    pXB->psaddr->pSerId = *((long *) sp);
	    pXB->chid = pXB->logid = *((long *) sp+1);	/* get `stdout' */
            if(nc == 3)
            {
                long dc = *((long *)sp+2);
                if(dc != pXB->psaddr->pSerId)
                {
                    pXB->logid = dc;
                }
            }
	    sp += (nc << 2);	        /* skip channels */
            nc = *((short *)sp)++;
            if(msp = malloc(nc + 1))
            {
                memcpy(msp, sp, nc);
                *(msp+nc) = 0;
                if(dirn = destack(&msp))
                {       
		    if(dirn == SEND)
		    {
			MakeFileList (msp);
			p = xfl;
		    }
		    else
		    {
			p = msp;
		    }
		    
                    XPRUse(MyProtocol);
                    res = XPRIO (p, dirn);
                    io_sbyte(pXB->psaddr->pSerId, 100, '\r');
                }
            }
	}
    }
    return res;
}

int (*_Cstart) (int, char **) = main;
